from os.path import basename
from subprocess import call
import json

import requests

class LoginFailed(Exception):
    pass

class VerifyFailed(Exception):
    pass

class Website:
    _base              = "https://www.humblebundle.com"
    _url_login         = "https://www.humblebundle.com/login"
    _url_processlogin  = "https://www.humblebundle.com/processlogin"
    _url_humbleguard   = "https://www.humblebundle.com/user/humbleguard"
    _url_home          = "https://www.humblebundle.com/home"
    _url_order_details = "https://www.humblebundle.com/api/v1/order/"
    _url_bundle        = "https://www.humblebundle.com/downloads?key="
    
    def __init__(self, cookies=None):
        self._session = requests.Session()
        if cookies is not None:
            self._session.cookies = cookies
        
        try:
            self._session.cookies.load()
        except IOError:
            pass
    
    def login_start(self, email, password):
        login = self._session.get(self._url_login)
        content = login.text
        csrf_begin = content.find("value='", content.find("csrftoken")) + len("value='")
        csrf_end   = content.find("'", csrf_begin)
        csrf = content[csrf_begin:csrf_end]
        
        payload = {
            "_le_csrf_token": csrf,
            "authy-token"   : "",
            "goto"          : "/home",
            "password"      : password,
            "qs"            : "",
            "submit-data"   : "",
            "username"      : email,
        }
        
        login = self._session.post(self._url_processlogin, data=payload)
        try:
            answer = login.json()
        except ValueError:
            raise LoginFailed(login.status_code)
        if not answer["success"]:
            if "username" in answer["errors"]:
                raise LoginFailed(answer["errors"]["username"][0])
            if "captcha" in answer["errors"]:
                raise LoginFailed(answer["errors"]["captcha"][0])
            raise LoginFailed(answer["errors"])
        
        home = self._session.get(self._base + login.json()["goto"])
        if home.text.find("Account Protection"):
            return False
        else:
            return True
    
    def login_verify(self, code):
        payload = {
            "code": code,
            "goto": "",
            "qs"  : "",
        }
        
        verify = self._session.post(self._url_humbleguard, data=payload)
        
        try:
            if not verify.json()["success"]:
                raise VerifyFailed(verify.json()["errors"]["code"][0])
        except ValueError:
            if verify.status_code != 200:
                raise VerifyFailed(verify.status_code)
    
    def login_end(self):
        self._session.cookies.save()
    
    def is_logged(self):
        return self._logged
    
    def get_gamekeys(self):
        home = self._session.get(self._url_home)
        home = home.text
        fragment = home[home.find("var gamekeys ="):]
        fragment = fragment[fragment.find("["):]
        fragment = fragment[:fragment.find("]")+1]
        return json.loads(fragment)
    
    def _get_order(self, key):
        url = self._url_order_details + key
        order = self._session.get(url)
        order = json.loads(order.text)
        
        return order
    
    def get_order_details(self, key):
        order = self._get_order(key)
        
        return self._format_order(order)
    
    def _format_order(self, order):
        out = {}
        out["gamekey"]      = order["gamekey"]
        out["machine_name"] = order["product"]["machine_name"]
        out["human_name"]   = order["product"]["human_name"]
        out["category"]     = order["product"]["category"]
        out["total"]        = order["total"]
        out["created"]      = order["created"]
        out["url"]          = self._url_bundle + order["gamekey"]

        # remove useless ZERO WIDTH SPACE characters
        out["human_name"] = out["human_name"].translate({
            '\u200b': None
        })
        
        products = order["subproducts"]
        return out, products
    
    def get_products_details(self, products):
        for product in products:
            yield self._format_product(product)
    
    def _format_product(self, product):
        out = {}
        out["machine_name"]        = product["machine_name"]
        out["human_name"]          = product["human_name"]
        out["icon"]                = product["icon"]
        out["url"]                 = product["url"]
        out["editor_human_name"]   = product["payee"]["human_name"]
        out["editor_machine_name"] = product["payee"]["machine_name"]
        
        # remove useless ZERO WIDTH SPACE characters
        out["human_name"] = out["human_name"].translate({
            ord('\u200b'): None
        })
        
        downloads = product["downloads"]
        return out, downloads
    
    def get_downloads_details(self, downloads):
        for filelist in downloads:
            for download in self._format_download(filelist):
                yield download
    
    def _format_download(self, download):
        for file in download["download_struct"]:
            if "asm_config" in file:
                continue
            
            out = {}
            out["platform"]     = download["platform"]
            out["machine_name"] = download["machine_name"]
            
            # build a nice code name
            out["name"] = out["machine_name"] + "-" + file["name"].translate({
                ord(u" "): None,
                ord(u"."): None,
                ord(u"("): None,
                ord(u")"): None
            })
            
            out["title"]       = file["name"]
            out["human_size"]  = file.get("human_size", None)
            out["size"]        = file.get("file_size", None)
            out["sha1"]        = file.get("sha1", "")
            out["md5"]         = file.get("md5", "")
            
            if "url" in file:
                out["url"]     = file["url"].get("web", None)
                out["torrent"] = file["url"].get("bittorrent", None)
            elif "external_link" in file:
                out["url"]     = file["external_link"]
                out["torrent"] = None
            
            if "url" in out and out["url"]:
                out["filename"] = basename(out["url"]).split("?", 1)[0].lstrip()
            else:
                continue
                # out["filename"] = None
            
            yield out
    
    def find_url(self, key, product_name, searched_filename, torrent=False):
        order = self._get_order(key)
        
        products = order["subproducts"]
        for product in products:
            if product["machine_name"] == product_name:
                break
        
        downloads = product["downloads"]
        for download in downloads:
            for file in download["download_struct"]:
                if "url" not in file:
                    continue
                http_url    = file["url"].get("web", None)
                torrent_url = file["url"].get("bittorrent", None)
                filename    = basename(http_url).split("?", 1)[0]
                if filename == searched_filename:
                    if torrent:
                        return http_torrent
                    else:
                        return http_url
        
        return None
    
if __name__ == "__main__":
    import os.path
    from http.cookiejar import LWPCookieJar
    humble = Website(cookies=LWPCookieJar(os.path.expanduser("~/.cache/humble/cookies.txt")))
    
    for key in humble.get_gamekeys():
        order, products = humble.get_order_details(key)
        for (k,v) in order.items():
            print(k, ":", v)
        
        for product, downloads in humble.get_products_details(products):
            for (k,v) in product.items():
                print("    ", k, ":", v)
            downloads = humble.get_downloads_details(downloads)
            for download in downloads:
                for (k,v) in download.items():
                    print("        ", k, ":", v)

